﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("CircularKernel", "A circular kernel function")]
  public class CircularKernel : KernelBase {

    #region HLConstructors & Boilerplate
    [StorableConstructor]
    protected CircularKernel(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }
    protected CircularKernel(CircularKernel original, Cloner cloner) : base(original, cloner) { }
    public CircularKernel() {
      Parameters.Add(new FixedValueParameter<DoubleValue>(BetaParameterName, "The beta in the kernel function 2*pi*(acos(-d)-d*(1-n²)^(0.5)) where n = ||x-c|| and d = n/beta", new DoubleValue(2)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CircularKernel(this, cloner);
    }
    #endregion

    protected override double Get(double norm) {
      if (Math.Abs(Beta) < double.Epsilon) return double.NaN;
      if (norm >= Beta) return 0;
      var d = norm / Beta;
      return Math.Acos(-d) - d * Math.Sqrt(1 - d * d) - Math.PI / 2;
    }

    protected override double GetGradient(double norm) {
      if (Math.Abs(Beta) < double.Epsilon) return double.NaN;
      if (Beta < norm) return 0;
      return -2*Math.Pow(norm,3)/(Math.Pow(Beta,4)*Math.Sqrt(1-norm*norm/(Beta*Beta)));
    }
  }
}
